import React, { Component } from 'react';
import PropTypes from 'prop-types';
import CodeMirror from 'react-codemirror2';

const SOURCE = 'codeMirror';

const sendErr = (text, key = '[react-collaborative-codemirror]') => {
    throw new Error(`${key} ${text}`);
};

class ReactCollaborativeCodemirror extends Component {
    constructor(props) {
        super(props);
        if (!this.props.shareDoc) {
            sendErr('ShareDoc is not initialized');
        }
        if (!this.props.contentKey) {
            sendErr('Content key must be specified');
        }
        this.state = {
            value: (this.props.shareDoc.data && this.props.shareDoc.data[this.props.contentKey]) || ''
        };
    }

    createOp(codeMirror, change) {
        const op = [];
        const startLine = change.from.line;
        let textIndex = 0;
        for (let i = 0; i < startLine; i += 1) {
            textIndex += codeMirror.lineInfo(i).text.length + 1;
        }

        textIndex += change.from.ch;
        if (textIndex > 0) {
            op.push(textIndex);
        }

        if (change.to.line !== change.from.line || change.to.ch !== change.from.ch) {
            let delLen = 0;
            const numLinesRemoved = change.removed.length;
            for (let i = 0; i < numLinesRemoved; i += 1) {
                delLen += change.removed[i].length + 1;
            }
            delLen -= 1;
            op.push({ d: delLen });
        }

        if (change.text) {
            const text = change.text.join('\n');
            if (text) {
                op.push(text);
            }
        }
        return op;
    }

    createDocOp(op, key) {
        return [{
            p: [key],
            t: 'text',
            o: op
        }];
    }

    handleStart() {
        this.props.shareDoc.subscribe((err) => {
            if (err) throw err;
            if (this.props.shareDoc.type) {
                this.setState({
                    value: this.props.shareDoc.data[this.props.contentKey]
                });
            }
            this.props.shareDoc.on('op', (op, source) => {
                if (source !== SOURCE) {
                    this.setState({ value: this.props.shareDoc.data[this.props.contentKey] });
                }
            });
        });
    }

    handleChange(codeMirror, change) {
        const op = this.createOp(codeMirror, change);
        const docOp = this.createDocOp(op, this.props.contentKey);
        this.props.shareDoc.submitOp(docOp, { source: SOURCE });
    }

    render() {
        return (
            <div>
                <CodeMirror
                    value={this.state.value}
                    editorDidMount={(codeMirror) => {
                        this.handleStart(codeMirror);
                    }}
                    onChange={(codeMirror, metadata) => {
                        this.handleChange(codeMirror, metadata);
                    }}
                    options={this.props.codeMirrorOptions}
                />
            </div>
        );
    }
}


ReactCollaborativeCodemirror.propTypes = {
    codeMirrorOptions: PropTypes.object,
    shareDoc: PropTypes.object.isRequired,
    contentKey: PropTypes.string.isRequired
};

export default ReactCollaborativeCodemirror;
